@@ -1686,6 +1686,22 @@ get_clipboard_mime(PyObject *self UNUSED, PyObject *args) {
     if (PyErr_Occurred()) return NULL;
     Py_RETURN_NONE;
 }
+
+static PyObject*
+make_x11_window_a_dock_window(PyObject *self UNUSED, PyObject *args UNUSED) {
+    int x11_window_id;
+    PyObject *dims;
+    if (!PyArg_ParseTuple(args, "iO!", &x11_window_id, &PyTuple_Type, &dims)) return NULL;
+    if (PyTuple_GET_SIZE(dims) != 12 ) { PyErr_SetString(PyExc_TypeError, "dimesions must be a tuple of length 12"); return NULL; }
+    if (!glfwSetX11WindowAsDock) { PyErr_SetString(PyExc_RuntimeError, "Failed to load glfwGetX11Window"); return NULL; }
+    uint32_t dimensions[12];
+    for (Py_ssize_t i = 0; i < 12; i++) dimensions[i] = PyLong_AsUnsignedLong(PyTuple_GET_ITEM(dims, i));
+    if (PyErr_Occurred()) return NULL;
+    glfwSetX11WindowAsDock(x11_window_id);
+    glfwSetX11WindowStrut(x11_window_id, dimensions);
+    Py_RETURN_NONE;
+}
+
 // Boilerplate {{{
 
 static PyMethodDef module_methods[] = {
@@ -1704,6 +1720,7 @@ static PyMethodDef module_methods[] = {
     METHODB(x11_display, METH_NOARGS),
     METHODB(get_click_interval, METH_NOARGS),
     METHODB(x11_window_id, METH_O),
+    METHODB(make_x11_window_a_dock_window, METH_VARARGS),
     METHODB(strip_csi, METH_O),
 #ifndef __APPLE__
     METHODB(dbus_send_notification, METH_VARARGS),
